//+------------------------------------------------------------------+
//|                                             Display CSV file.mq4 |
//+------------------------------------------------------------------+

#property  indicator_chart_window

#include <hanover --- function header (np).mqh>

extern   string   FileName                   = "myfxoutlook_mt4dash.txt";
extern   string   FontNameSizeColor          = "Consolas,12,White";        // Font Name, Size, Color
extern   string   WindowCornerHposVposVspace = "0,TR,10,20,16";                 // Window, Corner, Start Hpos, Start Vpos, Vert Spacing
extern   string   Visibility                 = "M1,M5,M15,M30,H1,H4,D1,W1,MN";
extern   string   RefreshPeriod              = "+0";
extern   int      BGWidth                    = 25;
extern   string   BGColor                    = "b64r0g0";

double     buffer0[];

string     arr[10], IndiName, FontName;
int        vis, RefreshEveryXMins, Window, Corner, HorizPos, VertPos, VertSpacing, FontSize;
color      FontColor;
bool       FirstTime;
datetime   prev_time;

//+------------------------------------------------------------------+
int deinit()  {
//+------------------------------------------------------------------+
  del_obj();
  return(0);
}

//+------------------------------------------------------------------+
void del_obj()  {
//+------------------------------------------------------------------+
  int k=0;
  while (k<ObjectsTotal())   {
    string objname = ObjectName(k);
    if (StringSubstr(objname,0,StringLen(IndiName)) == IndiName)  
      ObjectDelete(objname);
    else
      k++;
  }    
  return(0);
}

//+------------------------------------------------------------------+
int init()  {
//+------------------------------------------------------------------+
  FileName = StringReplace(FileName,"[sym]",Symbol());
  FileName = StringReplace(FileName,"[tf]",Period());
  FileName = StringReplace(FileName,"[dt]",DateToStr(TimeCurrent(),"Y-M-D_H-I-S"));
  IndiName = "Display_CSV[" + FileName + "]";
  IndicatorShortName("Display_CSV: "+FileName);

  StrToStringArray(WindowCornerHposVposVspace,arr);
  Window      = StrToInteger(arr[0]);
  Corner      = 2*(StringFind(StringUpper(arr[1]),"B")>=0) + (StringFind(StringUpper(arr[1]),"R")>=0);
  HorizPos    = StrToInteger(arr[2]);
  VertPos     = StrToInteger(arr[3]);
  VertSpacing = StrToInteger(arr[4]);

  StrToStringArray(FontNameSizeColor,arr);
  FontName    = arr[0];
  FontSize    = StrToInteger(arr[1]);
  FontColor   = StrToColor(arr[2]);

  RefreshEveryXMins = StrToTF(RefreshPeriod);
  prev_time = -9999;
  vis = GetVisibility(Visibility);
  Window = WindowFind(IndiName);
  if (Window < 0)
    Window = WindowsTotal();
  Window = 0;
  del_obj();
  plot_obj();
  return(0);
}

//+------------------------------------------------------------------+
int start()  {
//+------------------------------------------------------------------+
  if (RefreshEveryXMins < 0)  {
    if (FirstTime)  {
      del_obj();
      plot_obj();
    }
    FirstTime = false;      
    return(0);
  }  
  if (RefreshEveryXMins == 0) {
    del_obj();
    plot_obj();    
  } else {
    if (prev_time != iTime(Symbol(),RefreshEveryXMins,0))  {
      del_obj();
      plot_obj();
      prev_time = iTime(Symbol(),RefreshEveryXMins,0);
  } }      
  return(0);
}

//+------------------------------------------------------------------+
int plot_obj()  {
//+------------------------------------------------------------------+
  if (StringFind(Visibility+",",TFToStr(Period())+",")<0)
    return(0);
  int h = FileOpen(FileName, FILE_CSV|FILE_READ,'~');
  if (h==0) {
    Comment("File "+FileName+" not found.");
    return(0);
  }  

  // First pass loads data from file into arrays dt, val   (i.e. date/time, and value)
  for (int c=0; !FileIsEnding(h); c++)  {
    string tmp = FileReadString(h);
    if (FileIsEnding(h))  break;
//    if (StringLen(StringTrimRight(tmp)) < 1)   continue;    // ignore blank lines
    if (BGWidth>0)
      PlotLabel (IndiName+NumberToStr(c,"'-'Z4'b'"),  true,  Window, Corner, HorizPos, VertPos+c*VertSpacing, StringRepeat("g",BGWidth), StrToColor(BGColor), VertSpacing, "Webdings", 0.0, false, vis);      // Plot text label
    PlotLabel (IndiName+NumberToStr(c,"'-'Z4"),     false, Window, Corner, HorizPos, VertPos+c*VertSpacing,  tmp,   FontColor, FontSize, FontName,  0.0, false, vis);       // Plot text label
  }
  FileClose(h);
  
  return(0);
}

//+------------------------------------------------------------------+
#include <hanover --- extensible functions (np).mqh>

