//+------------------------------------------------------------------+
//|                                           Plot External Data.mq4 |
//+------------------------------------------------------------------+

#property  indicator_separate_window

#include <hanover --- function header (np).mqh>

#property  indicator_buffers 1
#property  indicator_color1  Red
#property  indicator_width1  1

extern   string   FileName         = "data.csv";
extern   string   DateFormat       = "YYYY.MM.DD HH:II";
extern   string   DateDelimiters   = ". :";
extern   string   FieldDelimiter   = ",";
extern   color    LineColor        = Red;
extern   int      LineWidth        = 1;
extern   int      LineStyle        = STYLE_SOLID;
extern   double   ScalingFactor    = 1.0;
extern   string   Visibility       = "M1,M5,M15,M30,H1,H4,D1,W1,MN";
extern   string   RefreshPeriod    = "+0";
extern   bool     ShowDataAtClose  = false;
extern   double   InvertThePlot    = 0;

double     buffer0[];

datetime   dt[9999], prev_time;
double     val[9999];
string     arr[4], IndiName;    // added to accomplish format conversion
int        vis, RefreshEveryXMins;
bool       FirstTime;

//+------------------------------------------------------------------+
int deinit()  {
//+------------------------------------------------------------------+
  del_obj();
  return(0);
}

//+------------------------------------------------------------------+
void del_obj()  {
//+------------------------------------------------------------------+
  int k=0;
  while (k<ObjectsTotal())   {
    string objname = ObjectName(k);
    if (StringSubstr(objname,0,StringLen(IndiName)) == IndiName)  
      ObjectDelete(objname);
    else
      k++;
  }    
  return(0);
}

//+------------------------------------------------------------------+
int init()  {
//+------------------------------------------------------------------+
  FileName = StringReplace(FileName,"[sym]",Symbol());
  FileName = StringReplace(FileName,"[tf]",Period());
  FileName = StringReplace(FileName,"[dt]",DateToStr(TimeCurrent(),"Y-M-D_H-I-S"));
  IndiName = "PED[" + FileName + "]-";
  SetIndexBuffer(0,buffer0);
  SetIndexStyle(0,DRAW_LINE,LineStyle,LineWidth,LineColor);
  SetIndexDrawBegin(0,0);
  IndicatorShortName("External: "+FileName);

  RefreshEveryXMins = StrToTF(RefreshPeriod);
  prev_time = -9999;
  vis = GetVisibility(Visibility);
  
  del_obj();
  plot_obj();
  return(0);
}

//+------------------------------------------------------------------+
int start()  {
//+------------------------------------------------------------------+
  if (RefreshEveryXMins < 0)  {
    if (FirstTime)  {
      del_obj();
      plot_obj();
    }
    FirstTime = false;      
    return(0);
  }  
  if (RefreshEveryXMins == 0) {
    del_obj();
    plot_obj();    
  } else {
    if (prev_time != iTime(Symbol(),RefreshEveryXMins,0))  {
      del_obj();
      plot_obj();
      prev_time = iTime(Symbol(),RefreshEveryXMins,0);
  } }      
  return(0);
}

//+------------------------------------------------------------------+
int plot_obj()  {
//+------------------------------------------------------------------+
  if (StringFind(Visibility+",",TFToStr(Period())+",")<0)
    return(0);
  int h = FileOpen(FileName, FILE_CSV|FILE_READ,'~');
  if (h==0) {
    Comment("File "+FileName+" not found.");
    return(0);
  }  

  // First pass loads data from file into arrays dt, val   (i.e. date/time, and value)
  for (int c=0; !FileIsEnding(h) && c<9999; c++)  {
    string tmp = FileReadString(h);
    if (FileIsEnding(h))  break;
    if (StringLen(StringTrimRight(tmp)) < 1)   continue;    // ignore blank lines
    StrToStringArray(tmp,arr,FieldDelimiter);
    dt[c]  = StrToDate(arr[0],DateFormat,DateDelimiters);
    val[c] = StrToNumber(arr[1]);
//    if (c>0)  log(c,DateToStr(dt[c]),NumberToStr(val[c],"RT-3.5"));    // debugging only
  }
  FileClose(h);
  
  // Second pass re-synchs data from dt, val arrays onto chart timeframe, and plots them accordingly
  c--;
  for (int i=0; i<=Bars; i++)  {
    datetime tim = Time[i];
    if (ShowDataAtClose && i>0)
      tim = Time[i-1]-1;
    if (ShowDataAtClose && i==0)
      tim = TimeCurrent();
    while(tim<dt[c]) c--;
    if (c>=0) {
      if (InvertThePlot>0)
        buffer0[i] = InvertThePlot - val[c]*ScalingFactor;
      else  
        buffer0[i] = val[c]*ScalingFactor;
     }   
//    if (i<30) log(i,DateToStr(Time[i]),c,DateToStr(dt[c]),val[c],buffer0[i]);    // debugging only
  }
  
  return(0);
}

//+------------------------------------------------------------------+
#include <hanover --- extensible functions (np).mqh>

